/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CWaiter.h  -  description
                             -------------------
    begin                : Mon Apr 17 2000
    copyright            : (C) 2000-2005 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it 
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at 
   your option) any later version.

   This library is distributed in the hope that it will be useful, but 
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/
#ifndef __CWAITER_H__
#define __CWAITER_H__

#ifndef _WIN32

#include <unistd.h>
#include <pthread.h>

#else

#include <windows.h>
#include <process.h>

#endif

#include <sptk3/sptk.h>

/// @addtogroup utility Utility Classes
/// @{

/// Synchronization object
class SP_EXPORT CWaiter {

protected:

#ifndef _WIN32
   pthread_mutex_t   m_waiter;				///< Mutex object
   pthread_cond_t    m_condition;			///< Condition object
#else
   HANDLE            m_waiter;				///< Event object
#endif

public:

   /// Constructor
   CWaiter();

   /// Destructor
   ~CWaiter();

   /// Try to lock waiter. Blocks until the lock.
   void lock();

   /// Unlocks the waiter.
   void unlock();

   /// Waits for a signal, timeout in msec
   /// @param timeout int, timeout in milliseconds 
   int  waitForSignal(int timeout=-1);   

   /// Sends signal to stop waiting.
   /// Caller needs to aquire lock explicitly before calling this.
   void sendSignalNoLock();

   /// Sends signal to stop waiting
   void sendSignal();

   /// Sends signal to all waiting threads.
   /// Caller needs to aquire lock explicitly before calling this.
   void broadcastSignalNoLock();

   /// Sends signal to all waiting threads.
   void broadcastSignal();
};

/// @}

#endif
